"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const readYaml = require("js-yaml");
const writeYaml = require("yamljs");
const vscode = require("vscode");
const path = require("path");
const fs = require("fs");
const Profiles_1 = require("../Profiles");
const ZoweTreeNode_1 = require("../abstract/ZoweTreeNode");
const utils_1 = require("../utils");
const nls = require("vscode-nls");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
const LINKS_FOLDER = "profile_links";
const FILE_SUFFIX = ".yaml";
function getLinkedProfile(node, type, logger) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            if (node instanceof ZoweTreeNode_1.ZoweTreeNode) {
                return findLinkedProfile(yield getProfile(node), type);
            }
            throw new Error(localize(0, null));
        }
        catch (err) {
            if (logger) {
                logger.warn(err.message);
            }
            throw (err);
        }
    });
}
exports.getLinkedProfile = getLinkedProfile;
function getProfile(node) {
    if (node instanceof ZoweTreeNode_1.ZoweTreeNode) {
        return node.getProfile();
    }
    throw new Error(localize(1, null));
}
exports.getProfile = getProfile;
function linkProfileDialog(aProfile) {
    return __awaiter(this, void 0, void 0, function* () {
        let chosenName;
        let chosenType;
        if (aProfile) {
            const possibles = Profiles_1.Profiles.getInstance().getAllTypes().filter((value) => value !== aProfile.type);
            const quickPickOptions1 = {
                placeHolder: localize(2, null),
                ignoreFocusOut: true,
                canPickMany: false
            };
            chosenType = yield vscode.window.showQuickPick(possibles, quickPickOptions1);
            if (chosenType) {
                const profiles = Profiles_1.Profiles.getInstance().getNamesForType(chosenType);
                const quickPickOptions2 = {
                    placeHolder: localize(3, null),
                    ignoreFocusOut: true,
                    canPickMany: false
                };
                chosenName = yield vscode.window.showQuickPick(profiles, quickPickOptions2);
                if (chosenName) {
                    try {
                        yield saveLinkedProfile(aProfile, chosenType, chosenName);
                        vscode.window.showInformationMessage(localize(4, null, chosenType, chosenName, aProfile.type, aProfile.name));
                    }
                    catch (err) {
                        vscode.window.showErrorMessage(localize(5, null) + err.message);
                    }
                }
            }
        }
    });
}
exports.linkProfileDialog = linkProfileDialog;
function findLinkedProfile(aProfile, type) {
    return __awaiter(this, void 0, void 0, function* () {
        let profile;
        if (aProfile) {
            const linkRootDirectory = path.join(utils_1.getZoweDir(), LINKS_FOLDER);
            if (!fs.existsSync(linkRootDirectory)) {
                fs.mkdirSync(linkRootDirectory);
            }
            const file = path.join(linkRootDirectory, aProfile.type, aProfile.name + FILE_SUFFIX);
            if (fs.existsSync(file)) {
                const properties = readYaml.safeLoad(fs.readFileSync(file));
                const links = properties.secondaries;
                for (const element of Object.keys(links)) {
                    if (element === type) {
                        try {
                            profile = yield Profiles_1.Profiles.getInstance().directLoad(type, links[type]);
                        }
                        catch (err) {
                            throw new Error(localize(6, null) + " + " + err.message);
                        }
                    }
                }
            }
        }
        return profile;
    });
}
function saveLinkedProfile(primary, secondaryType, secondaryName) {
    return __awaiter(this, void 0, void 0, function* () {
        const secondaryArray = {};
        let content = {
            secondaries: secondaryArray,
        };
        if (primary) {
            let targetfile = path.join(utils_1.getZoweDir(), LINKS_FOLDER);
            if (!fs.existsSync(targetfile)) {
                fs.mkdirSync(targetfile);
            }
            targetfile = path.join(targetfile, primary.type);
            if (!fs.existsSync(targetfile)) {
                fs.mkdirSync(targetfile);
            }
            targetfile = path.join(targetfile, primary.name + FILE_SUFFIX);
            if (fs.existsSync(targetfile)) {
                content = readYaml.safeLoad(fs.readFileSync(targetfile));
            }
            if (!content || !(content instanceof Object)) {
                content = {
                    secondaries: secondaryArray,
                };
            }
            else if (!content.secondaries) {
                content.secondaries = secondaryArray;
            }
            content.secondaries[`${secondaryType}`] = secondaryName;
            fs.writeFileSync(targetfile, writeYaml.stringify(content));
        }
    });
}

//# sourceMappingURL=profileLink.js.map
